@@ -1,7 +1,7 @@
 /***** BEGIN LICENSE BLOCK *****
 
     FlashGot - a Firefox extension for external download managers integration
-    Copyright (C) 2004-2011 Giorgio Maone - g.maone@informaction.com
+    Copyright (C) 2004-2013 Giorgio Maone - g.maone@informaction.com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -279,13 +279,12 @@ FlashGotDM.prototype = {
 ,
   createJobBody: function(links) {
     var jobLines = [];
-    var postData = links.postData || "";
  
     for (var j = 0, len = links.length, l; j < len; j++) {
       jobLines.push((l = links[j]).href,
            l.description,
            this.getCookie(l, links),
-           postData);
+           l.postData || links.postData ||  '');
       this.updateProgress(links, j, len);
     }
     return jobLines.join("\n");
@@ -376,7 +375,7 @@ FlashGotDM.prototype = {
     var l, parts;
     for (j = links.length; j-- > 0;) {
       l = links[j];
-      parts = l.href.match(/http[s]{0,1}:\/\/([^\/]+\.[^\/]+)/i); // host?
+      parts = l.href.match(/^https?:\/\/([^\/:]+)/i); // host?
       if (parts) {
         host = parts[1];
         var hpos = host.indexOf("@");
@@ -447,7 +446,7 @@ FlashGotDM.prototype = {
     
     const f = fg.tmpDir.clone();
     f.append("cookies");
-    f.createUnique(0, 0600);
+    f.createUnique(0, B8('600'));
     IO.writeFile(f, cookies.join("\n"));
     
     fg.doomPrivateFile(f);
@@ -458,7 +457,7 @@ FlashGotDM.prototype = {
   createJobFile: function(job) {
     const jobFile = fg.tmpDir.clone();
     jobFile.append("flashgot.fgt");
-    jobFile.createUnique(0, 0700);
+    jobFile.createUnique(0, B8('700'));
     IO.writeFile(jobFile, job);
     return jobFile;
   }
@@ -503,7 +502,7 @@ FlashGotDM.prototype = {
     var bos = null;
     try {
       const fos = CC["@mozilla.org/network/file-output-stream;1"].createInstance(CI.nsIFileOutputStream);
-      fos.init(exeFile, 0x02 | 0x08 | 0x20, 0755, 0);
+      fos.init(exeFile, 0x02 | 0x08 | 0x20, B8('755'), 0);
       bos = CC['@mozilla.org/binaryoutputstream;1'].createInstance(CI.nsIBinaryOutputStream);
       bos.setOutputStream(fos);
       bos.writeBytes(templateImage, bytesCount);
@@ -607,7 +606,7 @@ FlashGotDM.prototype = {
       ? links.length + " @ " +
       (function() {
         var ll = links.map(function(l) {
-          return l.href.match(/^[\w-]+:\/*([^\/]+)/)[1];
+          return l.href.match(/^[\w-]+:\/*([^\/]*)/)[1];
         }).sort(); // extract and sort domains
         for (var j = ll.length; j-- > 1;) // remove dupes
           if (ll[j - 1] == ll[j]) ll.splice(j, 1);
@@ -618,13 +617,14 @@ FlashGotDM.prototype = {
     var params = {
       title: "FlashGot (" + this.name.replace(/[\(\)]/g, "") + ") " + urlLabel,
       initialDir: initialDir,
-      choosenDir: null
+      choosenDir: null,
+      links: links
     }
     
     this.getWindow().openDialog(
               "chrome://flashgot/content/chooser.xul",
               "flashgotChooser",
-              "chrome, dialog, modal, dependent, centerscreen",
+              "chrome, dialog, modal, dependent, centerscreen, resizable",
               params);
     
     if (params.choosenDir) {
@@ -669,13 +669,16 @@ FlashGotDM.prototype = {
 
 
 // *** Unix-like DMS ***********************************************************
-function FlashGotDMX(name, cmd, argsTemplate) {
+function FlashGotDMX(name, cmd, argsTemplate, unixPath) {
   if (arguments.length != 0) {
     this._init(name);
-    const cmds = FlashGotDMX.prototype.unixCmds;
-    cmds[name] = cmd;
+    this.unixCmds[name] = cmd;
     this.unixCmd = cmd;
     if (argsTemplate) this.argsTemplate = argsTemplate;
+    if (unixPath) {
+      this.unixPath = unixPath;
+      this.unixPaths.push(unixPath);
+    }
     this.cookieSupport =  /\[.*?(?:CFILE|COOKIE).*?\]/.test(this.argsTemplate);
   }
   if (fg.isMac) {
@@ -688,6 +691,7 @@ FlashGotDMX.prototype.exeName = "flashgot.sh";
 FlashGotDMX.prototype.terminal = false;
 FlashGotDMX.prototype.askPath = [true, true, true];
 FlashGotDMX.prototype.unixCmds = {};
+FlashGotDMX.prototype.unixPaths = [];
 FlashGotDMX.prototype.__defineGetter__("unixShell", function() {
   var f = CC["@mozilla.org/file/local;1"].createInstance(CI.nsILocalFile);
   try {
@@ -707,8 +711,7 @@ FlashGotDMX.prototype.__defineGetter__("unixShell", function() {
 FlashGotDMX.prototype.argsTemplate = "[URL]";
 FlashGotDMX.prototype.launchSupportTest = function(testFile) {
   const cmds = this.unixCmds;
-  var script="(\n";
-
+  var script= "PATH=\"" + this.unixPaths.join(":") + ":$PATH:" +  "\"\n(\n";
   for (var name in cmds) {
     cmd = cmds[name];
     script += " which \"" + cmd + "\" && echo '"
@@ -734,7 +737,7 @@ FlashGotDMX.prototype.shellEsc = function(s) {
   return s ? s.replace(/([\\\*\?\[\]\$&<>\|\(\)\{\};"'`])/g,"\\$1").replace(/\s/g,"\\ ") : null;
 };
 FlashGotDMX.prototype.createJob = function(links, opType) {
-  const shellEsc = this.shellEsc;
+    const shellEsc = this.shellEsc;
   // basic implementation
 
   const folder = shellEsc(links.folder);
@@ -744,22 +747,8 @@ FlashGotDMX.prototype.createJob = function(links, opType) {
   
   
   var len = links.length;
-  var job;
-  if (this.terminal) {
-    const title = shellEsc("FlashGot " +  (len > 1 ? "(" + len + ")" : links[0].href));
-    const shell = this.unixShell && shellEsc(this.unixShell.path) || '';
-    job = 
-      "if [ \"$1\" = \"\" ]; then\n" +
-      "  if [ \"$COLORTERM\" = \"gnome-terminal\" ] && which gnome-terminal >/dev/null 2>&1; then\n" +
-      "    gnome-terminal -t " + title + " -x " + shell + " \"$0\" term && exit\n" +
-      "  fi\n" +
-      "  if which xterm >/dev/null 2>&1; then\n" +
-      "    xterm -T " + title + " -e " + shell + " \"$0\" term && exit\n" +
-      "  fi\n" +
-      "fi\n"
-  } else job = "";
-  
-  job += this.shreddingCmd();
+  var job = (this.unixPath ? "PATH=" + this.unixPath + ":$PATH\n" : "") +
+            this.shreddingCmd();
   
   if (links.folder) job += "cd '" + links.folder + "'\n";
     
@@ -791,7 +780,7 @@ FlashGotDMX.prototype.createJob = function(links, opType) {
       COOKIE: shellEsc(this.getCookie(l, links)),
       CFILE: cookieFile,
       FOLDER: folder, 
-      POST: postData,
+      POST: l.postData && shellEsc(l.postData) || postData,
       UFILE: ufile,
       ULIST: ulist,
       USERPASS: l.userPass && shellEsc(l.userPass) || null,
@@ -800,6 +789,29 @@ FlashGotDMX.prototype.createJob = function(links, opType) {
     this.updateProgress(links, j, len);
   }
   
+  if (this.terminal) {
+    const autoClose = fg.getPref("term.autoClose", -1);
+    const title = shellEsc("FlashGot " +  (len > 1 ? "(" + len + ")" : links[0].href));
+    const shell = this.unixShell && shellEsc(this.unixShell.path) || '';
+    job = 
+      "if [ -z \"$FLASHGOT_TERM\" ]; then\n" +
+      "  export FLASHGOT_TERM; FLASHGOT_TERM=1\n" +
+      "  if which flashgot-term >/dev/null 2>&1; then\n" +
+      "     flashgot-term " + title + " " + shell + " \"$0\" && exit\n" +
+      "  fi\n" +
+      "  if which gnome-terminal >/dev/null 2>&1; then\n" +
+      "    gnome-terminal -t " + title + " -x " + shell + " \"$0\" && exit\n" +
+      "  fi\n" +
+      "  if which xterm >/dev/null 2>&1; then\n" +
+      "    xterm -T " + title + " -e " + shell + " \"$0\" && exit\n" +
+      "  fi\n" +
+      "fi\n" +
+      job +
+        (autoClose < 0
+          ? "\necho -n 'Press any key to exit...' && read k || sleep 5\n"
+          : "\nsleep " + autoClose + "\n");
+  }
+  
   return job;
 };
 
@@ -814,7 +826,7 @@ FlashGotDMX.prototype.performJob = function(job, blocking) {
   job = "#!" + this.unixShell.path + "\n" + job + this.shreddingCmd();
   
   const jobFile = this.createJobFile(job);
-  jobFile.permissions = 0700;
+  jobFile.permissions = parseInt("0700", 8);
   return !(fg.isMac 
     ? this.runNative([], blocking, FlashGotDMMac.exeFile)
     : this.runNative([jobFile.path], blocking, this.unixShell)
@@ -904,7 +916,7 @@ FlashGotDMMac.prototype.createExecutable = function() {
    }
    if (mustCreate) {
       this.log("Creating Mac executable");
-      exeFile.create(0, 0700);
+      exeFile.create(0, B8('700'));
       IO.writeFile(exeFile, scriptLauncher);
      
       try {
@@ -917,7 +929,7 @@ FlashGotDMMac.prototype.createExecutable = function() {
       }
    }
    this.log("Setting executable permissions on " + exeFile.path);
-   exeFile.permissions = 0700;
+   exeFile.permissions = parseInt("0700", 8);
 
    return mustCreate;
   } catch(ex) {
@@ -964,7 +976,7 @@ FlashGotDMMac.prototype.createJobFile = function(job) {
     jobFile.remove(true);
   } catch(ex) {}
   try {
-    jobFile.create(0, 0600);
+    jobFile.create(0, B8('600'));
     IO.writeFile(jobFile, job, /^#/.test(job) ? null : fg.getPref("appleScriptEncoding"));
     return jobFile;
   } catch(ex) {
@@ -990,9 +1002,12 @@ FlashGotDMMac.prototype.createJob = function(links,opType) {
 
 
 // *** Custom DMS **************************************************************
-function FlashGotDMCust(name) {
+function FlashGotDMCust(name, codeName) {
   if (arguments.length == 0 || (!name) || (!name.length)) return;
   name = name.replace(/,/g, " ");
+  if (codeName) {
+    this._codeName = codeName;
+  }
   this._init(name);
   this.prefsBase = "custom." + this.codeName + ".";
 }
@@ -1182,8 +1197,9 @@ FlashGotDMCust.prototype.shellEsc = function(s) {
 
 FlashGotDMCust.prototype.winEscHack = function(s) {
   // hack for bug at http://mxr.mozilla.org/seamonkey/source/xpcom/threads/nsProcessCommon.cpp#149 
+  var rstr = /^\w+:\/+/.test(s) ? "# " : " ";
   return (/[;&=]/.test(s) && !/\s/.test(s)) // "=" and ";" are command line separators on win!!!
-    ? s + " " : s; // we add a space to force escaping
+    ? s + rstr : s; // we add a space to force escaping
 }
 
 FlashGotDMCust.prototype.performJob = function(job) {
@@ -1222,7 +1238,7 @@ FlashGotDMCust.prototype.performJob = function(job) {
         REFERER: referrer, 
         COOKIE: esc(this.getCookie(l, links)), 
         FOLDER: folder, 
-        POST: postData,
+        POST: l.postData && esc(l.postData) || postData,
         CFILE: cookieFile,
         UFILE: urls.file,
         ULIST: urls.list,
@@ -1349,17 +1365,19 @@ FlashGotDM.initDMS = function() {
   new FlashGotDM("Download Master");
   
   for each (dm in [new FlashGotDM("DTA"), new FlashGotDM("DTA (Turbo)")]) {
-    dm.__defineGetter__("supported", 
-      function() {
-        try {
-          IOS.newChannel("chrome://dta-modules/content/support/filtermanager.js", null, null);
-          return true;
-        } catch(e) {}
-        try {
-          IOS.newChannel("resource://dta/support/filtermanager.jsm", null, null);
-          return true;
-        } catch(e) {}
-        return  "dtaIFilterManager" in CI || "@downthemall.net/privacycontrol;1" in CC 
+    dm.__defineGetter__("_supported", function() {
+      delete this._supported;
+      return this._supported = (function() {
+            try {
+              IOS.newChannel("chrome://dta-modules/content/support/filtermanager.js", null, null);
+              return true;
+            } catch(e) {}
+            try {
+              IOS.newChannel("resource://dta/support/filtermanager.jsm", null, null);
+              return true;
+            } catch(e) {}
+            return  "dtaIFilterManager" in CI || "@downthemall.net/privacycontrol;1" in CC;
+          })();
     });
     dm.turboDTA = /Turbo/.test(dm.name);
     dm.nativeUI = dm.turboDTA
@@ -1437,10 +1455,13 @@ FlashGotDM.initDMS = function() {
               url: wrapURL(l.href, cs),
               description: l.description,
               ultDescription: '',
-              referrer: referrer
+              referrer: referrer,
+              isPrivate: fg.inPrivate,
           });
           
-          if ("fname" in l) {
+          if ("postData" in l) lastItem.postData = l.postData;
+          
+          if (("fname" in l) && l.fname) {
             lastItem.fileName = lastItem.destinationName = l.fname;
             // if (single && !turbo) single = false;
           }
@@ -1471,7 +1492,7 @@ FlashGotDM.initDMS = function() {
             DTA.saveSingleItem(w, turbo, lastItem);
           } else if (single && (DTA.saveSingleLink || w.DTA_AddingFunctions.saveSingleLink)) {
             this.log("DTA.saveSingleLink " + lastItem.url);
-            DTA.saveSingleLink(w, turbo, lastItem.url, lastItem.referrer, lastItem.description, links.postData);
+            DTA.saveSingleLink(w, turbo, lastItem.url, lastItem.referrer, lastItem.description, lastItem.postData || links.postData);
           } else {
             this.log("DTA.saveLinkArray " + anchors.toSource());
             DTA[turbo ? "turboSaveLinkArray" : "saveLinkArray"](w, anchors, images);
@@ -1606,7 +1627,7 @@ FlashGotDM.initDMS = function() {
         }
         var file = fg.tmpDir.clone();
         file.append("flashgot.grx");
-        file.createUnique(0,0600);
+        file.createUnique(0, B8('600'));
         var charset=null;
         try {
           charset=fg.getPref("GetRight.charset",
@@ -1771,7 +1792,7 @@ FlashGotDM.initDMS = function() {
             rg.setInfo(l.description);
             rg.setCookie(this.getCookie(l, links));
             rg.setReferer(ref);
-            rg.setPostData(links.postData);
+            rg.setPostData(l.postData || links.postData);
             rg.setConfirmation(true);
             rg.addDownload();
             return;
@@ -2002,6 +2023,7 @@ FlashGotDM.initDMS = function() {
         referrer.host = "localhost";
         chan.referrer = referrer;
       }
+      chan.loadFlags != chan.INHIBIT_PERSISTENT_CACHING;
     } catch(e) {
       fg.log(e);
     }
@@ -2174,6 +2196,11 @@ FlashGotDM.initDMS = function() {
     return FlashGotDMX.prototype.createJob.call(this, links, opType)
   } 
   
+  dm = new FlashGotDMX("FlareGet", "flareget",
+      "[URL] [-o FNAME] [-c COOKIE] [-l CFILE] [-a USERPASS] [-u UA] [-r REFERER] [-d POST]",
+      "/opt/flareget");
+  dm.askPath = ASK_NEVER;
+ 
   dm = new FlashGotDMX("KDE KGet","kget", "[ULIST]");
   dm.askPath = ASK_NEVER;
   
@@ -2215,6 +2242,9 @@ FlashGotDM.initDMS = function() {
   dm = new FlashGotDMX("FatRat", "fatrat", "[ULIST]");
   dm.askPath = ASK_NEVER;
   
+  dm = new FlashGotDMX("MultiGet", "multiget", '[url=URL] [refer=REFERER]');
+  dm.askPath = ASK_NEVER;
+  
   dm = new FlashGotDMX("Prozilla", "proz", '-r [-P FOLDER] [URL]');
   dm.terminal = true;
   
@@ -2241,8 +2271,10 @@ FlashGotDM.initDMS = function() {
     return this.__proto__.createJob.call(this, links, opType);
   }
   
+  var ugetTemplate = '[--http-cookie-file=CFILE] [--http-post-data=POST] [--http-referer=REFERER] [--filename=FNAME] [--input-file=UFILE]';
   if (fg.isWindows) {
-    dm = new FlashGotDMCust("Uget");
+    dm = new FlashGotDMCust("uGet", "Uget");
+    dm.argsTemplate = ugetTemplate;
     dm.cookieSupport = true;
     dm.askPath = ASK_NEVER;
     dm.custom = false;
@@ -2271,16 +2303,183 @@ FlashGotDM.initDMS = function() {
     
     
   } else {
-    dm = new FlashGotDMX("Uget", "uget-gtk", '[--http-cookie-file=CFILE] [--http-post-data=POST] [--http-referer=REFERER] [--input-file=UFILE] [URL]');
+    dm = new FlashGotDMX("uGet", "uget-gtk",  ugetTemplate);
+    dm._codeName = "Uget";
     dm.postSupport = true;
     dm.askPath = ASK_NEVER;
-  } 
+  }
+  // backward compatibility with name change
+  // TODO: refactor me for future reuse
+  if (/\bUget\b/.test(fg.getUPref('detect.cache', ''))) { 
+    if (fg.defaultDM === dm.codeName) fg.defaultDM = dm.name;
+    if (fg.getUPref("media.dm") === dm.codeName) fg.setUPref("media.dm", dm.name);
+    fg.setUPref('detect.cache', fg.getUPref('detect.cache', '').replace(/\bUget\b/, dm.name));
+  }
+  
+  
+  var xdmTemplate = '[-u URL] -m [-c COOKIE] [-r REFERER]';
+  if (fg.isWindows) {
+    dm = new FlashGotDMCust("XDM");
+    dm.argsTemplate = xdmTemplate;
+    dm.cookieSupport = true;
+    dm.askPath = ASK_NEVER;
+    dm.custom = false;
+    dm._supported = null;
+    
+    dm.customSupportCheck = function() {
+      var path;
+      try {
+        path = this.readWinRegString("CURRENT_USER", "SOFTWARE\\xdm", "Path");
+      } catch (e) {
+        path = this.getPref("path");
+      }
+        
+      if (!path) return false;
+      
+      try {  
+        var exeFile = CC["@mozilla.org/file/local;1"].createInstance(CI.nsILocalFile);
+        exeFile.initWithPath(path);
+        if (!(exeFile.exists() && exeFile.isExecutable())) return false;
+        this.exeFile = exeFile;
+        return true;
+      } catch(e) {
+        return false;
+      } 
+    };
+    
+    
+  } else {
+    dm = new FlashGotDMX("XDM", "xdman",  xdmTemplate);
+    dm.askPath = ASK_NEVER;
+  }
+  
+  
+  dm = new FlashGotDM("pyLoad");
+  dm.askPath = ASK_NEVER;
+  dm.cookieSupport = true;
+  dm.autoselect = false;
+  dm._supported = true; // pyload does not necessarily need to run on localhost, so it should always be active
+  dm._createRequest = function(method, url, callback, async) {
+    var r = CC["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(CI.nsIXMLHttpRequest);
+    r.open(method, url, async);
+    try {
+      var chan = r.channel;
+      if (chan instanceof CI.nsIHttpChannel) {
+        var referrer = chan.URI.clone();
+        referrer.host = "localhost";
+        chan.referrer = referrer;
+      }
+    } catch(e) {
+      fg.log(e);
+    }
+    
+    if (callback) {
+      r.addEventListener("readystatechange", function() {
+        if (r.readyState === 4) {
+          try {
+            fg.log("pyLoad response status:\n" + r.status + "\n" + r.statusText);
+          } catch (e) {}
+          callback(r);
+        }
+      }, false);
+    }
+    return r;
+  }
+  dm.performDownload = function(links, opType) {
+    const pp = { urls: [], descriptions: [], cookies: [] };
+    var l, j, len;
+    for (j = 0, len = links.length; j < len; j++) {
+      l = links[j];
+      pp.urls.push(l.href);
+      pp.descriptions.push(l.description);
+      pp.cookies.push(this.getCookie(l, links));
+    }
+    
+    if (links.some(function(l) { return "fname" in l }))
+      pp.fnames = links.map(function(l) { return l.fname || ''; });
+    
+    if (links.some(function(l) { return "pwd" in l }))
+      pp.arcpass = links.map(function(l) { return l.pwd || ''; });
+    
+    if (links.some(function(l) { return "userPass" in l }))
+      pp.httpauth = links.map(function(l) { return l.userPass || '' });
+    
+    var pkg = (links.document && links.document.title || "FlashGot");
+    // filter out all non-ASCII characters, because pyLoad 0.4.9 crashes when logging a non-ASCII character (e.g. '»')
+    pkg = encodeURIComponent(pkg.replace(new RegExp("[^\x20-\x7E]","im"),""));
+
+    const data = [
+      "autostart=" + (this.getPref("autostart", true) ? "1" : "0"),
+      "package=" + pkg
+    ];
+    const referrer = this.getReferrer(links);
+    if (referrer) data.push("referer=" + encodeURIComponent(referrer));
+    if (links.postData) data.push("postData=" + encodeURIComponent(links.postData));
+    for (j in pp) {
+      data.push(j + "=" + encodeURIComponent(pp[j].join("\n")));
+    }
+    this._post(data.join("&"));
+  };
+  dm._post = function(data) {
+    var self = this;
+    var url = this.getPref("url");
+    var r = this._createRequest("POST", url, function(r) {
+      if (r.status != 200) {
+      	var msg = r.status === 500
+          ? "An internal error occured during adding the package to pyLoad. Check the pyLoad logfile for further information!"
+          : "pyLoad is not responding on " + self.getPref("url")
+            + "!\nPlease check your firewall settings and the pyLoad configuration for the 'ClickAndLoad' plugin.\n"
+            + "Or try to restart pyLoad or the device itself (pyLoad may not be responding properly after some time)."
+        ;
+        DOM.mostRecentBrowserWindow.alert(msg);
+      }
+    });
+
+    r.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
+    r.send(data);
+  };
   
   dm = new FlashGotDMX("Aria 2", "aria2c", '--continue [-d FOLDER] [-o FNAME] [--referer=REFERER] [--load-cookies=CFILE] [--input-file=UFILE] [URL]');
   dm.terminal = true;
   
   dm = new FlashGotDMX("Steadyflow", "steadyflow", '[add URL]');
   dm.askPath = ASK_NEVER;
+  
+  
+    if (fg.isWindows) {
+        dm = new FlashGotDMCust("ZigzagDownLoader");
+        dm.cookieSupport = true;
+        dm.custom = false;
+        dm._supported = null;
+       
+        dm.customSupportCheck = function() {
+          var path;
+          try {
+            path = this.readWinRegString("LOCAL_MACHINE", "SOFTWARE\\Cygwin\\setup", "rootdir");
+          } catch (e) {
+            path = null;
+          }
+           
+          if (!path) return false;
+         
+          try { 
+            var exeFile = CC["@mozilla.org/file/local;1"].createInstance(CI.nsILocalFile);
+            exeFile.initWithPath(path);
+            exeFile.append("zdl.bat");
+            if (!(exeFile.exists() && exeFile.isExecutable())) return false;
+            
+            this.exeFile = exeFile;
+            this.argsTemplate = "--stream [URL] [FNAME] [FOLDER] [CFILE] [COOKIE] [REFERER]";
+            return true;
+          } catch(e) {
+            return false;
+          }
+        };
+      } else {
+        dm = new FlashGotDMX("ZigzagDownLoader", "zdl", "--stream [URL] [FNAME] [FOLDER] [CFILE] [COOKIE] [REFERER]");
+      }
+
+
 
   if (fg.isMac) {
     dm = new FlashGotDMX("Progressive Downloader", "$(defaults read com.PS.PSD psAppPath)", "-add [url UFILE] [cookie CFILE] [referer REFERER] [destination FOLDER]");
@@ -2312,9 +2511,9 @@ FlashGotDM.initDMS = function() {
       '" to AddURL {"' + urlList.join('","') + '"}';
     
     if (this.postSupport) {
-
-      if (links.postData) { 
-        job +=' with form data "' + links.postData + '"';
+      var postData = links[0].postData || links.postData;
+      if (postData) { 
+        job +=' with form data "' + postData + '"';
       }
 
       const referer = this.getReferrer(links);
@@ -2355,8 +2554,9 @@ FlashGotDM.initDMS = function() {
     
     var job = 'tell app "' + this.macAppName + '" to download URLs {"'
       + urlList.join('", "') + '"}';
-    if (links.postData) {
-      job += ' by posting data "' + links.postData + '"';
+    var postData = links.links[0].postData || links.postData;
+    if (postData) {
+      job += ' by posting data "' + postData + '"';
     }
     job += ' to POSIX path "' + links.folder + '"';
     if (cookieList.length) {
@@ -2405,17 +2605,25 @@ FlashGotDM.initDMS = function() {
       cookies.push(this.getCookie(l, links));
       titles.push(l.description.replace(/[\u0000-\u0020"\s]+/g, ' '));
     }
-    var job = "tell application \""+ this.macAppName + "\"\n" +
+    var job = "tell application \""+ this.macAppName + "\"\nactivate\n" +
       'add URLs {"' + urls.join('", "') + 
       "\"} with referrer \"" + this.getReferrer(links) +
       "\" with cookies {\"" + cookies.join('", "') +
       "\"} with titles {\"" + titles.join('", "') +
       "\"} ";
-    if (links.postData)
-      job += 'with post data "' + links.postData + '"';
-    return job + " from applecation \"FlashGot\"\nend tell\n";
+    var postData = links[0].postData || links.postData;
+    if (postData) {
+      job += 'with post data "' + postData + '"';
+    }
+    return job + "\nend tell\n";
   }
   
+  dm = new FlashGotDMMac("Folx 3", "com.eltima.Folx3");
+  ["nativeUI", "cookieSupport", "createJob"].forEach(function(p) {
+    dm[p] = FlashGotDM.dms["Folx"][p];
+  });
+  
+  
   if ("nsIDownloadManager" in CI) {
     dm = new FlashGotDM(fg.getString("dm.builtIn"));
     dm._codeName = "_Built_In_";
@@ -2441,7 +2649,7 @@ FlashGotDM.initDMS = function() {
         l = links[j];
         l.uri = IOS.newURI(l.href, cs, null);
         
-        if (noRedir || l.fname || !(l.uri instanceof CI.nsIURL) ||
+        if (noRedir || l.postData || l.fname || !(l.uri instanceof CI.nsIURL) ||
             rx && rx.test(l.uri.fileName)) {
           count--;
           continue;
@@ -2449,12 +2657,19 @@ FlashGotDM.initDMS = function() {
         
         allDone = false;
         
-        ch = IOS.newChannelFromURI(l.uri);
+        try {
+          ch = IOS.newChannelFromURI(l.uri);
+        }
+        catch (x) {
+          this.log("newChannelFromURI() failed: " + l.href + " (" + l.uri.spec + "): " + x + "\n" + x.stack);
+          --count;
+          continue;
+        }
         ch.asyncOpen({
           dm: this,
           link: l,
           onStartRequest: function(ch, ctx) {
-            if (!(ch instanceof CI.nsIHttpChannel) || Math.round(ch.responseStatus / 100) != 3)
+            if (!(ch instanceof CI.nsIHttpChannel) || !Components.isSuccessCode(ch.status) || Math.floor(ch.responseStatus / 100) != 3)
               this._done(ch);
           },
           onDataAvailable: function(req, ctx , stream , offset , count ) {
@@ -2464,14 +2679,13 @@ FlashGotDM.initDMS = function() {
           },
 
           _done: function(ch) {
+            var status = ch.status;
             ch.cancel(NS_BINDING_ABORTED);
-            if (ch instanceof CI.nsIHttpChannel && ch.responseStatus == 200 && !/\b(?:x|ht)ml\b/.test(ch.contentType)) {
+            if (ch instanceof CI.nsIHttpChannel && Components.isSuccessCode(status) && ch.responseStatus == 200 && !/\b(?:x|ht)ml\b/.test(ch.contentType)) {
               // this.link.uri = ch.URI; // removed, it breaks filesonic.com...
-              try {
-                this.link.fname = ch.getResponseHeader("content-disposition").match(/; filename="([^"]+)/i)[1];
-              } catch(e) {}
+              this.link.fname = fg.extractFileName(ch) || this.link.fname;
             }
-            
+           
             if (--count === 0) {
               callback();
               progressHolder.progress.update(100);
@@ -2494,6 +2708,18 @@ FlashGotDM.initDMS = function() {
       this._checkLinks(links, cs, function() { self.performDownload(links, opType); });
     };
     
+    dm._streamify = function(s, contentType) {
+      if (!s) return null;
+      var stream = CC["@mozilla.org/io/string-input-stream;1"].createInstance(CI.nsIStringInputStream);
+      stream.setData(s, s.length);
+      if (stream instanceof CI.nsISeekableStream) stream.seek(0, 0);
+      var mis = CC["@mozilla.org/network/mime-input-stream;1"].createInstance(CI.nsIMIMEInputStream);
+      if (contentType) { mis.addHeader("Content-Type", contentType); }
+      mis.addContentLength = true;
+      mis.setData(stream);
+      return mis;
+    }
+    
     dm.performDownload = function(links, opType) {
       if (!links._prepared) {
         links._prepared = true;
@@ -2506,26 +2732,37 @@ FlashGotDM.initDMS = function() {
         CI.nsIWebBrowserPersist.PERSIST_FLAGS_FROM_CACHE;
       
       const dType = CI.nsIDownloadManager.DOWNLOAD_TYPE_DOWNLOAD;
-      var postData = links.postStream || null;
-
+      var postData = links.postData, postStream;
       var uri, folder, file;
-      var persist, args;
+      var args;
       var now = Date.now() * 1000;
       var dm = CC["@mozilla.org/download-manager;1"].getService(CI.nsIDownloadManager);
       folder = CC["@mozilla.org/file/local;1"].createInstance(CI.nsILocalFile);
       folder.initWithPath(links.folder);
       var mozAddDownload;
-      if(dm.startBatchUpdate) {
-        mozAddDownload = typeof(dType) == "undefined" 
-          ? function(src, dest, des, persist) { return dm.addDownload(src, dest, des, null, now, null, persist); }
-          : function(src, dest, des, persist) { return dm.addDownload(dType, src, dest, des, null, null, now, null, persist); }
-          ;
-        dm.startBatchUpdate();
-      } else {
-        mozAddDownload = function(src, dest, des, persist) { return dm.addDownload(dType, src, dest, des, null, now, null, persist); };
+      try { // Fx 26 and above
+        Components.utils.import("chrome://flashgot/content/JSDownloadsAPI.jsm");
+      } catch (e) {
+        fg.log(e);
+        JSDownloadsAPI = null;
       }
+      if (!JSDownloadsAPI) {
+        // older browsers
+        if(dm.startBatchUpdate) {
+          mozAddDownload = typeof(dType) == "undefined" 
+            ? function(src, dest, des, persist) { return dm.addDownload(src, dest, des, null, now, null, persist); }
+            : function(src, dest, des, persist) { return dm.addDownload(dType, src, dest, des, null, null, now, null, persist); }
+            ;
+          dm.startBatchUpdate();
+        } else {
+          mozAddDownload = function(src, dest, des, persist) { return dm.addDownload(dType, src, dest, des, null, now, null, persist, fg.inPrivate); };
+        }
+      }
+      
       var dl;
       const overwrite = this.getPref("overwrite", false);
+      var persist = {}; // dummy for JSDownloadsAPI
+      var isPrivate = fg.inPrivate;
       for(var j = 0, len = links.length, l; j < len; j++) {
         l = links[j];
         try {
@@ -2534,22 +2771,49 @@ FlashGotDM.initDMS = function() {
           file = fg.createDownloadFile(folder, l.fname || uri, overwrite);
           
           if (!file) continue;
-          
-          persist = CC["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"].createInstance(CI.nsIWebBrowserPersist);
-          persist.persistFlags = persistFlags;
-          
           fg.log("Saving " + l.href + " to " + file.path);
           
-          persist.progressListener = dl = 
-            mozAddDownload(uri, IOS.newFileURI(file), file.leafName, persist)
-              .QueryInterface(CI.nsIWebProgressListener);
+          if (JSDownloadsAPI) {
+            
+            JSDownloadsAPI.add(l, links, file.path, isPrivate, function(err) {
+              if (isPrivate) {
+                Cu.reportError(err);
+              } else {
+                fg.log("Skipping link " + l.href + ": " + err);
+              }
+            });
+            
+            continue;
+          }
+
+          persist = CC["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"].createInstance(CI.nsIWebBrowserPersist);
+          persist.persistFlags = persistFlags;
+          dl =  mozAddDownload(uri, IOS.newFileURI(file), file.leafName, persist);
+          if (dl instanceof CI.nsIWebProgressListener) persist.progressListener = dl;
+          else continue;
           
-          persist.saveURI(uri, l.cacheKey || null,
-                  links.refURI, l.postStream || postData || null, null, file);
+          postStream = this._streamify(l.postData || postData, l.postContentType);
+          var extraHeaders = "";
+          if (l.extraHeaders) {
+            for (var p in l.extraHeaders) {
+              extraHeaders += p + ": " + l.extraHeaders[p] + "\r\n";
+            }
+          }
+          if (!extraHeaders) { extraHeaders = null; }
+          if ("savePrivacyAwareURI" in persist) { // Gecko >= 19
+              persist.savePrivacyAwareURI(uri, l.cacheKey || null,
+                  links.refURI, postStream, extraHeaders, file, fg.inPrivate);
+          } else {
+            persist.saveURI(uri, l.cacheKey || null,
+                  links.refURI, postStream, extraHeaders, file);
+          }
         } catch (e) {
           fg.log("Skipping link " + l.href + ": " + e);
         }
       }
+      
+      if (JSDownloadsAPI) return;
+      
       if(dm.endBatchUpdate) dm.endBatchUpdate();
       if(dm.flush) dm.flush();
       
