@@ -1,7 +1,7 @@
 /***** BEGIN LICENSE BLOCK *****
 
     FlashGot - a Firefox extension for external download managers integration
-    Copyright (C) 2004-2011 Giorgio Maone - g.maone@informaction.com
+    Copyright (C) 2004-2013 Giorgio Maone - g.maone@informaction.com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -84,33 +84,65 @@ HttpInterceptor.prototype = {
     return true;
   }
 ,
+  _slurp: function(stream, len) {
+    var sis = CC['@mozilla.org/binaryinputstream;1'].createInstance(CI.nsIBinaryInputStream);
+    sis.setInputStream(stream);
+    return sis.readBytes(len);
+  },
   extractPostData: function(channel, res) {
     res = res || {};
-    res.postData = res.postStream = null;
+    res.postData = null;
     if (channel instanceof CI.nsIUploadChannel &&
        channel.uploadStream instanceof CI.nsISeekableStream) {
       this.log("Extracting post data...");
-      var sis;
       try {
-        res.postStream = channel.uploadStream;
-        res.postStream.seek(0, 0);
-        sis = CC['@mozilla.org/scriptableinputstream;1'].createInstance(CI.nsIScriptableInputStream);
-        sis.init(res.postStream);
-        var postData  = sis.read(sis.available());
-        res.postStream.seek(0, 0);
+        var stream = channel.uploadStream;
+        var originalOffset = stream.tell(); 
+        stream.seek(2 /* EOF */, 0);
+        var size = stream.tell();
+        // Extra paranoia mode.
+        if (size !== originalOffset) {
+          this.log("WARNING: original offset != size: " + originalOffset + ", " + size);
+        }
+        stream.seek(0, 0);
+        var s = this._slurp(stream, size);
+        stream.seek(0, originalOffset);
         
-        // buffered persistent copy 
-        res.postStream = CC["@mozilla.org/io/string-input-stream;1"].createInstance(CI.nsIStringInputStream);
-        res.postStream.setData(postData, postData.length);
-        if (res.postStream instanceof CI.nsISeekableStream) res.postStream.seek(0, 0);
+        // extract upload content type
+        const headerVisitor = {visitHeader: function(name, value) {
+          if ("content-type" === name.toLowerCase()) res.postContentType = value;
+        }};
+        channel.visitRequestHeaders(headerVisitor);
+
+        // For some reason, grooveshark.com
+        // puts the "Referer", "Content-Type" and "Content-Length" headers
+        // into channel.uploadStream instead of the headers (visitRequestHeaders()).
+        // The download manager still has to send the "Range: bytes=0-"
+        // header in order to get the file.
+        if (/\.grooveshark\.com$/.test(channel.URI.asciiHost)) {
+          // Remove the headers.
+          const hdrEndPos = s.indexOf("\r\n\r\n");
+          if (hdrEndPos !== -1) {
+            var headers = s.substr(0, hdrEndPos).split(/\r\n/);
+            var m;
+            for each (var hdr in headers) {
+              m = hdr.match(/^(.+?)\s*:\s*(.+)$/);
+              if (!m) continue;
+              headerVisitor.visitHeader(m[1], m[2]);
+              if (res.postContentType) break;
+            }
+            if (!res.extraHeaders) res.extraHeaders = {};
+            res.extraHeaders["Range"] = "bytes=0-";
+            s = s.substr(hdrEndPos + 4);
+          }
+        }
         
-        // remove headers
-        postData = postData.replace(/\s+$/,'').split(/[\r\n]+/)
-        res.postData = postData[postData.length - 1];
+        res.postData = s;
       } catch(ex) {
-        this.log(ex.message);
-      } finally {
-        if (sis) sis.close();
+        this.log(ex
+          ? (ex.message || ex) + "\n" + (ex.stack || new Error().stack)
+          : "Error extracting POST data\n" + new Error().stack
+        );
       }
     }
     return res;
@@ -139,10 +171,11 @@ HttpInterceptor.prototype = {
     
     this.log("Intercepting download...");
 
-    const pathParts=uri.path.split(/\//);
+    const fname = fg.extractFileName(channel) || uri.path.split(/\//).pop();
     var links = [ {
-     href: channel.URI.spec, 
-     description: pathParts[pathParts.length-1],
+     href: uri.spec,
+     fname: fname,
+     description: fname,
      noRedir: true
     } ];
     
@@ -154,9 +187,7 @@ HttpInterceptor.prototype = {
     }
     
     try {
-        links.document = channel.notificationCallbacks.QueryInterface(
-            CI.nsIInterfaceRequestor).getInterface(
-            CI.nsIDOMWindow).document;
+        links.document = DOM.findChannelWindow(channel).document;
         links.browserWindow = DOM.getChromeWindow(links.document.defaultView.top);
         if (links.browserWindow.wrappedJSObject) links.browserWindow = links.browserWindow.wrappedJSObject;
       } catch(e) {
