@@ -1,7 +1,7 @@
 /***** BEGIN LICENSE BLOCK *****
 
     FlashGot - a Firefox extension for external download managers integration
-    Copyright (C) 2004-2011 Giorgio Maone - g.maone@informaction.com
+    Copyright (C) 2004-2013 Giorgio Maone - g.maone@informaction.com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -78,7 +78,7 @@ var Module = {
       const catman = this.categoryManager;
       for (var j = 0, len = SERVICE_CATS.length; j < len; j++) {
         catman.addCategoryEntry(SERVICE_CATS[j],
-          SERVICE_CTRID, SERVICE_CTRID, true, true);
+          SERVICE_CTRID, SERVICE_CTRID, false, true);
       }
       this.firstTime = false;
       try {
@@ -96,7 +96,7 @@ var Module = {
       ).unregisterFactoryLocation(SERVICE_CID, fileSpec);
     const catman = this.categoryManager;
     for (var j = 0, len = SERVICE_CATS.length; j < len; j++) {
-      catman.deleteCategoryEntry(SERVICE_CATS[j], SERVICE_CTRID, true);
+      catman.deleteCategoryEntry(SERVICE_CATS[j], SERVICE_CTRID, false);
     }
   },
 
