@@ -1,7 +1,7 @@
 /***** BEGIN LICENSE BLOCK *****
 
     FlashGot - a Firefox extension for external download managers integration
-    Copyright (C) 2004-2011 Giorgio Maone - g.maone@informaction.com
+    Copyright (C) 2004-2013 Giorgio Maone - g.maone@informaction.com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@ var g_referrerRadio = null;
 var g_referrerText = null;
 var g_tmpDir = null;
 var g_wellGetPath = null;
+var g_pyLoadUrl = null;
 
 var g_cust = fg && {
   dummy: fg.createCustomDM(""),
@@ -231,6 +232,9 @@ function fgo_onload() {
       checkbox.checked = inverse ? !val : val;
     }
   );
+
+  g_pyLoadUrl = $("pyLoad-url");
+  g_pyLoadUrl.value = fg.getPref("dmsopts.pyLoad.url");
   
   g_referrerRadio=$("referrer-radio");
   g_referrerText=$("referrer-text");
@@ -263,7 +267,7 @@ function fgo_populateDMS(name) {
   const mediaList = $("media-dmList");
   
   var menuItem = mediaList.selectedItem; 
-  var mediaDM = menuItem ? menuItem.value && menuItem.label : fg.getPref("media.dm", ""); 
+  var mediaDM = menuItem ? menuItem.value && menuItem.label : fg.getUPref("media.dm", ""); 
   
   const map = g_cust.map;
   var dm;
@@ -454,7 +458,7 @@ function fgo_save() {
   
   var mediaList = $("media-dmList");
   var menuItem = mediaList.selectedItem;
-  fg.setPref("media.dm", menuItem && menuItem.value ? menuItem.label : "");
+  fg.setUPref("media.dm", menuItem && menuItem.value ? menuItem.label : "");
   
   if (g_extDirty) {
     const extensions = [];
@@ -467,6 +471,8 @@ function fgo_save() {
     }
     fg.extensions = extensions;
   }
+
+  fg.setPref("dmsopts.pyLoad.url",g_pyLoadUrl.value);
   
   fg.setPref("autoReferrer",g_referrerRadio.value=="true");
   fg.setPref("fakeReferrer",g_referrerText.value);
@@ -498,11 +504,8 @@ function fgo_save() {
 
 function fgo_showLog() {
   try {
-    if (fg.logStream) fg.logFlush();
-    open(CC['@mozilla.org/network/io-service;1'
-      ].getService(CI.nsIIOService
-      ).newFileURI(fg.logFile).spec,
-         "_FlashGot_LOG_");
+    fg.logWatch();
+    open(fg.logURI.spec, "_FlashGot_LOG_");
   } catch(ex) { dump(ex.message); }
 }
 
