@@ -1,7 +1,7 @@
 /***** BEGIN LICENSE BLOCK *****
 
     FlashGot - a Firefox extension for external download managers integration
-    Copyright (C) 2004-2011 Giorgio Maone - g.maone@informaction.com
+    Copyright (C) 2004-2013 Giorgio Maone - g.maone@informaction.com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,7 +20,8 @@
 ***** END LICENSE BLOCK *****/
 
 var gFlashGot = {
-    
+  _current_url: null,
+
   onload: function(ev) {
     ev.currentTarget.removeEventListener(ev.type, arguments.callee, false);
     try {
@@ -54,7 +55,9 @@ var gFlashGot = {
     if (target.tabContainer) {
         target.tabContainer.addEventListener("TabSelect", this.updateMediaStatus, false);
     }
-
+    
+    window.addEventListener("activate", this.updateMediaStatus, false);
+    
     target.addEventListener("load", this.updateMediaStatus, true);
     
     target.addEventListener("mousedown", function(ev) {
@@ -196,8 +199,6 @@ var gFlashGot = {
     
     window.setTimeout(function() {
       gFlashGotService.checkVersion();
-      if (gFlashGotService.firstRun) gFlashGot._mediaUI; // forces relocation
-      
       gFlashGot.toggleMainMenuIcon();
     }, 500);
   }
@@ -288,7 +289,8 @@ var gFlashGot = {
       toggleMenu("all", dis);
       toggleMenu("sel", dis || this.isSelInvalid);
       toggleMenu("buildGallery", false);
-      toggleMenu("media", dis || !this.media);
+      const media = this.media;
+      toggleMenu("media", dis || !(media && media.length));
       toggleMenu("opts", false);      
     }
   }
@@ -305,8 +307,8 @@ var gFlashGot = {
       var menuItem = document.getElementById("flashgot-menuitem-" + name);
       
       if(menuItem && 
-        ! ( menuItem.hidden = 
-            ( hidden || gFlashGotService.getPref("hide-" + name) ) )
+        !(menuItem.hidden = 
+            (hidden || gFlashGotService.getPref("hide-" + name)))
       ) {
        menuItem.setAttribute("disabled", disabled ? "true" : "false");
        if(! (menuItem.hidden=disabled && hideDisabled) ) {
@@ -342,7 +344,9 @@ var gFlashGot = {
     menuSwitch("tabs", (! (typeof(gBrowser)=="object" 
       && gBrowser.browsers && gBrowser.browsers.length > 1) ) 
                            || hidden || dm.disabledAll);
-    menuSwitch("media", hidden || !this.media);
+    
+    const media = this.media;
+    menuSwitch("media", hidden || !(media && media.length));
 
     hidden = false;
     menuSwitch("buildGallery", false);
@@ -429,7 +433,7 @@ var gFlashGot = {
           mi.setAttribute("class", "menuitem-iconic flashgot-command-menuitem");
           mi.setAttribute("label", dm.name);
           mi.setAttribute("id", id);
-          mi.setAttribute("oncommand", "gFlashGot.downloadSel(this.label) || gFlashGot.downloadPopupLink(this.label)");
+          mi.setAttribute("oncommand", "gFlashGot.downloadSel(this.label) || gFlashGot.downloadLink(this.label)");
           parentNode.insertBefore(mi, anchorNode);
         }
         dm.hideNativeUI(document);
@@ -471,7 +475,7 @@ var gFlashGot = {
         idPrefix += "tbb-";
         eventPostfix = "gFlashGot.downloadSel() || gFlashGot.downloadAll()";
       } else {
-        eventPostfix = "gFlashGot.downloadSel() || gFlashGot.downloadPopupLink()";
+        eventPostfix = "gFlashGot.download()";
       }
       lastMenu.setAttribute("hidden", "true");
       parentNode = lastMenu.parentNode;
@@ -513,47 +517,76 @@ var gFlashGot = {
   }
 ,
 
-  get _mediaUI() {
-    var addonBar = document.getElementById("addon-bar");
+  get _mediaIDs() {
     var widgetId = "flashgot-media-status";
-    if (addonBar) {
+    var addonBar = document.getElementById("addon-bar");
+    var statusBar = document.getElementById("status-bar");
+    if (addonBar || !document.getElementById("status-bar")) {
+      
       var el = document.getElementById(widgetId);
+      
       if (el) el.parentNode.removeChild(el);
       widgetId = "flashgot-media-tbb";
       el = document.getElementById(widgetId);
-      if (el && (el.nextSibling && el.nextSibling.id == "search-container" ||
-                 addonBar.collapsed && el.parentNode == addonBar)) {
+      
+      var addonBarVisible = addonBar && addonBar.boxObject.height;
+      if (el && (el.nextSibling && el.nextSibling.id === "search-container" ||
+                 addonBarVisible && el.parentNode === addonBar &&
+                 !gFlashGotService.getPref("media.iconInstalled", false))) {
         el = null;
       }
+  
       if (!el) {
-        var bar, refId;
-        var navBar = document.getElementById("nav-bar");
-        if(addonBar.collapsed && navBar && !navBar.collapsed) {
-          bar = navBar;
-          refId = "urlbar-container";
-        } else {
-          bar = addonBar;
-          refId = "status";
+        const fakeID = "flashgot-fake-tbb";
+        const splitRx = /\s*,\s*/;
+        var bar, set, refId;
+        if (addonBar) {
+          set = addonBar.currentSet.split(splitRx);
+          
+          if (set.indexOf(fakeID) === -1) {
+            set.push(fakeID);
+            
+            var navBar = document.getElementById("nav-bar");
+            
+            if(!addonBarVisible && navBar && !navBar.collapsed) {
+              addonBar.setAttribute("currentset", addonBar.currentSet = set.join(","));
+              document.persist(addonBar.id, "currentset");
+              bar = navBar;
+              set = bar.currentSet.split(splitRx);
+            } else {
+              bar = addonBar;
+              refId = "status";
+            }
+          }
         }
         
-        var set = bar.currentSet.split(/\s*,\s*/);
+        if (!bar) {
+          bar = document.getElementById("nav-bar");
+          set = bar && bar.currentSet.split(splitRx);
+        }
         
-        for (var p; (p = set.indexOf(widgetId)) > -1;)
-          set.splice(p, 1);
+        if (bar) {
+          for (var p; (p = set.indexOf(widgetId)) > -1;)
+            set.splice(p, 1);
           
-        set.splice(set.indexOf(refId), 0, widgetId); 
-        bar.setAttribute("currentset", bar.currentSet = set.join(","));
-        document.persist(bar.id, "currentset");
-        try {
-          window.BrowserToolboxCustomizeDone(true);
-        } catch (e) {}
+          var refIdx = refId ? set.indexOf(refId) : -1;
+          if (refIdx === -1) {
+            set.push(widgetId);
+          } else {
+            set.splice(refIdx, 0, widgetId);
+          }
+          bar.setAttribute("currentset", bar.currentSet = set.join(","));
+          document.persist(bar.id, "currentset");
+          try {
+            window.BrowserToolboxCustomizeDone(true);
+          } catch (e) {}
+          gFlashGotService.setPref("media.iconInstalled", true);
+        }
       }
     }
-    delete this._mediaUI;
-    return this._mediaUI =
-      [widgetId, "flashgot-menuitem-media", "flashgot-main-menuitem-media"]
-        .map(function(id) { return document.getElementById(id) })
-        .filter(function(el) { return !!el });
+  
+    delete this._mediaIDs;
+    return this._mediaIDs = [widgetId, "flashgot-menuitem-media", "flashgot-main-menuitem-media"]
   },
   
   _mediaTip: function(media) {
@@ -568,8 +601,9 @@ var gFlashGot = {
     var count = (media && media.length) ? " (" + media.length + ")" : "";
     var tip = this._mediaTip(media);
     
-    var l;
-    for each(var ui in this._mediaUI) {
+    var l, id, ui;
+    for each(id in this._mediaIDs) {
+      ui = document.getElementById(id);
       if (ui) {
         l = ui.getAttribute("label");
         if (l) ui.label = l.replace(/\s*\(\d+\)$/, '') + count;
@@ -614,8 +648,7 @@ var gFlashGot = {
     lastShowing: false,
     cycles: 0,
     get widget() {
-      delete this.widget;
-      return this.widget = gFlashGot._mediaUI[0];  
+      return document.getElementById(gFlashGot._mediaIDs[0]);  
     },
     run: function(once) {
       var w = this.widget;
@@ -644,25 +677,208 @@ var gFlashGot = {
   },
   
   prepareMediaMenu: function(menu) {
-    var pivot = Array.slice(menu.getElementsByTagName("menuseparator")).pop();
-    while (pivot.nextSibling) menu.removeChild(pivot.nextSibling);
+    var menu_parsed = document.getElementById("flashgot-media-parsed-popup");
+    var menu_parsed_dash = document.getElementById("flashgot-media-parsed-dash-popup");
+    var mi_refresh_decode_signature_func = document.getElementById("flashgot-media-parsed-refresh_signature_func");
+    this._clearMenu(document.getElementById("flashgot-media-parsed-menu").nextSibling);
+    this._clearMenu(mi_refresh_decode_signature_func.nextSibling);
+    this._clearMenu(menu_parsed_dash.firstChild);
+    menu_parsed.parentNode.setAttribute("hidden", "true");
+    menu_parsed_dash.parentNode.setAttribute("hidden", "true");
+    mi_refresh_decode_signature_func.setAttribute("hidden", "true");
     var m = this.media;
     if (!(m && m.length)) return false;
   
-    var mi;
+    // YouTube parsed streams. We'll sort them before building the menu.
+    var parsed_items = [], parsed_dash_items = [];
     for (var j = 0; j < m.length; j++) {
-      menu.appendChild(this._createMediaMenuItem(m[j]));
+      var mo = m[j];
+      // Special menu item, used for refreshing YouTube's signature decoding function.
+      if (mo.MediaSniffer && mo.originalURL === mo.MediaSniffer.YOUTUBE_REFRESH_SIGNATURE_FUNC_URL) {
+        var mi = mi_refresh_decode_signature_func;
+        mi.setAttribute("hidden", "false");
+        menu_parsed.parentNode.setAttribute("hidden", "false");
+        mi["flashgot::Youtube"] = mo.Youtube;
+        mi["flashgot::MediaSniffer"] = mo.MediaSniffer;
+        mi["flashgot::yt_win"] = mo.yt_win;
+        continue;
+      }
+      var mi = this._createMediaMenuItem(mo);
+      if (!mo.parsed) {
+        menu.appendChild(mi);
+        continue;
+      }
+      mi["flashgot::sort_key"] = mo.sort_key;
+      if (mo.yt_dash) {
+        mi["flashgot::yt_dash"] = mo.yt_dash;
+        parsed_dash_items.push(mi);
+      } else {
+        parsed_items.push(mi);
+      }
+    }
+    if (parsed_items.length || parsed_dash_items.length) {
+      function mo_cmp(lhs, rhs) { return lhs["flashgot::sort_key"] - rhs["flashgot::sort_key"]; }
+      if (parsed_items.length) {
+        parsed_items.sort(mo_cmp);
+        parsed_items.forEach(function(mi){delete mi["flashgot::sort_key"]; menu_parsed.appendChild(mi);});
+        menu_parsed.parentNode.setAttribute("hidden", "false");
+      }
+      if (parsed_dash_items.length) {
+        parsed_dash_items.sort(mo_cmp);
+        var show_av_separator = gFlashGotService.getPref("media.YouTube.dash.show_av_separator", true);
+        var prev_type = parsed_dash_items[0]["flashgot::yt_dash"];
+        parsed_dash_items.forEach(function(mi){
+          if (show_av_separator && prev_type !== mi["flashgot::yt_dash"]) {
+            show_av_separator = false;
+            menu_parsed_dash.appendChild(document.createElement("menuseparator"));
+          }
+          delete mi["flashgot::sort_key"];
+          delete mi["flashgot::yt_dash"];
+          menu_parsed_dash.appendChild(mi);
+        });
+        menu_parsed_dash.parentNode.setAttribute("hidden", "false");
+      }
+      this._cb_mediaMenuItem_key_down = false;
+      // https://developer.mozilla.org/en-US/docs/XUL/PopupGuide/PopupKeys:
+      // A key listener added to a <menupopup> will not receive any key events.
+      // Instead, you must add a capturing key listener to the document or window
+      // if you want to listen for keys pressed within a menu.
+      window.addEventListener("keydown", this._cb_mediaMenuItem_keydown, true);
+      window.addEventListener("keyup", this._cb_mediaMenuItem_keyup, true);
     }
     return true;
   },
+  unprepareMediaMenu: function() {
+    this._cb_mediaMenuItem_key_down = false;
+    window.removeEventListener("keydown", this._cb_mediaMenuItem_keydown, true);
+    window.removeEventListener("keyup", this._cb_mediaMenuItem_keyup, true);
+  },
+  _clearMenu: function(from) {
+    if (!from) { return; }
+    var pn = from.parentNode;
+    while (from.nextSibling !== null) { pn.removeChild(from.nextSibling); }
+    pn.removeChild(from);
+  },
   _createMediaMenuItem: function(l) {
-    mi = document.createElement("menuitem");
-    mi.setAttribute("label", l.label);
+    var mi = document.createElement("menuitem");
+
+    // Hilight the currently playing stream.
+    var is_current = false;
+    if (content._flashgotMediaCurrentUrl) {
+      is_current = l.Youtube
+        ? l.Youtube.stream_url_equals(content._flashgotMediaCurrentUrl, l.originalURL)
+        : content._flashgotMediaCurrentUrl === l.originalURL;
+    }
+    if (is_current) { mi.setAttribute("default", "true"); }
+
+    if (l.fallback_href) { mi.setAttribute("flashgotMediaFallbackHref", l.fallback_href); }
+    var label = l.label;
+    if (l.seek_pos && gFlashGotService.getPref("media.YouTube.seek_pos.show", true)) {
+      label = "@" + this._formatMediaSeekPos(l.seek_pos) + " " + label;
+    }
+    mi.setAttribute("label", label);
     mi.setAttribute("tooltiptext", l.tip);
-    mi.addEventListener("command", function() { gFlashGot.downloadMedia([l]); }, false);
+    // YouTube: hold Shift to download/copy fallback URL if it exists,
+    // otherwise the "main" URL will be used.
+    mi.addEventListener("command", function(ev) {
+      var o = l;
+      if (ev.shiftKey && l.fallback_href) {
+        o = Object.create(l);
+        o.href = l.fallback_href;
+      }
+      gFlashGot.downloadMedia([o]);
+    }, false);
+    mi.addEventListener("click", function(ev) {
+      if (ev.button === 2) {
+        Components.classes["@mozilla.org/widget/clipboardhelper;1"]
+          .getService(Components.interfaces.nsIClipboardHelper)
+          .copyString(ev.shiftKey && l.fallback_href || l.href);
+        ev.preventDefault();
+      }
+    }, true);
     return mi;
   },
-  
+  _cb_mediaMenuItem_key_down: false,
+  _cb_mediaMenuItem_keydown: function(ev) {
+    if (ev.keyCode !== 16 /*VK_SHIFT*/) { return; }
+    var This = gFlashGot;
+    if (This._cb_mediaMenuItem_key_down) { return; }
+    This._cb_mediaMenuItem_key_down = true;
+    var menu = document.getElementById("flashgot-media-parsed-popup");
+    var nl = menu.getElementsByTagName("menuitem");
+    for (var i = 0, len = nl.length; i !== len; ++i) {
+      var mi = nl[i];
+      mi.setAttribute("disabled", "" + !mi.hasAttribute("flashgotMediaFallbackHref"));
+    }
+  },
+  _cb_mediaMenuItem_keyup: function(ev) {
+    if (ev.keyCode !== 16 /*VK_SHIFT*/) { return; }
+    var This = gFlashGot;
+    if (!This._cb_mediaMenuItem_key_down) { return; }
+    This._cb_mediaMenuItem_key_down = false;
+    var menu = document.getElementById("flashgot-media-parsed-popup");
+    var nl = menu.getElementsByTagName("menuitem");
+    for (var i = 0, len = nl.length; i !== len; ++i) {
+      nl[i].setAttribute("disabled", "false");
+    }
+  },
+  _formatMediaSeekPos: function(ms) {
+    var h = Math.floor(ms / 1000 / 60 / 60);
+    var m = Math.floor((ms -= h * 60 * 60 * 1000) / 1000 / 60);
+    var s = Math.floor((ms -= m * 60 * 1000) / 1000);
+    ms -= s * 1000;
+    function strpad(str, pad_char, length) {
+      str = String(str);
+      var addon_len = length - str.length;
+      if (addon_len < 1) { return str; }
+      return new Array(addon_len + 1).join(pad_char) + str;
+    }
+    var rc;
+    // [h:m]m
+    if (gFlashGotService.getPref("media.YouTube.seek_pos.compact", true)) {
+      rc = "" + (h !== 0 ? h + ":" + strpad(m, "0", 2) : m);
+    }
+    // h:mm
+    else {
+      rc = "" + h + ":" + strpad(m, "0", 2);
+    }
+    // Seconds: ":ss".
+    rc += ":" + strpad(s, "0", 2);
+    // Milliseconds: ".SSS".
+    if (ms !== 0 && gFlashGotService.getPref("media.YouTube.seek_pos.show_ms", false)) {
+      rc += "." + strpad(ms, "0", 3);
+    }
+    return rc;
+  },
+  Youtube_refresh_signature: function(evt, elm) {
+    var win = elm['flashgot::yt_win'];
+    var MediaSniffer = elm['flashgot::MediaSniffer'];
+
+    // Remove the "Refresh signature" entry.
+    var media = this.media;
+    delete media._map[MediaSniffer.YOUTUBE_REFRESH_SIGNATURE_FUNC_URL];
+    var eidx = -1;
+    media.some(function(entry, idx){
+      if (entry.originalURL === MediaSniffer.YOUTUBE_REFRESH_SIGNATURE_FUNC_URL) {
+        eidx = idx;
+        return true;
+      }
+    });
+    if (eidx !== -1) { media.splice(eidx, 1); }
+
+    elm['flashgot::Youtube'].refresh_signature_func(win, function(){
+      // We're in a frame/iframe, and who knows how many other YouTube
+      // frames are also there (e.g. we're on a forum/blog), so, ideally,
+      // we should reload the top page for all those frames to get reloaded
+      // and rerequested by MediaSniffer.
+      // The problem is that I doubt everyone will like this reloading
+      // behavior.
+      //if (win !== win.top) { return win.top.location.reload(); }
+      MediaSniffer.checkYoutube(win, null, true);
+      }, true);
+    evt.preventDefault();
+    evt.stopPropagation();
+  },
 
   get srcWindow() {
     return document.commandDispatcher.focusedWindow;
@@ -696,7 +912,7 @@ var gFlashGot = {
     const doc = this.srcDocument;
     if(!doc) return 0;
     
-    var count = doc.links && doc.links.length || 0;
+    var count = doc.links ? doc.links.length || 0 : this.getXMLLinks(doc).length;
     
     if(gFlashGotService.getPref("includeImages")) 
        count += (doc.images && doc.images.length) || 0;
@@ -705,11 +921,24 @@ var gFlashGot = {
     return count;
   }
 ,
+  getXMLLinks: function(document) {
+    var nodesSnapshot = document.evaluate("//*/@*", document.documentElement, null,
+                                          XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
+    var links = [];
+    var href;
+    for (var j = 0, len = nodesSnapshot.snapshotLength; j < len; j++ ) {
+      href = nodesSnapshot.snapshotItem(j).nodeValue;
+      if (/^https?:/.test(href)) links.push({tagName: "XML", href: href, description: href });
+    }
+    return links;
+  },
   getLinks: function(filter, includeImages, doc) {
     if(typeof(doc) != "object") {
       doc = this.srcDocument;
     }
     if(doc == null) return [];
+    if (!doc.links) return this.getXMLLinks(doc);
+    
     const allLinks = [];
     
     function wrapAndFilter(newL, l) {
@@ -751,24 +980,23 @@ var gFlashGot = {
             l = elems[j];
             newL = wrap ? gFlashGot._wrapAnchor(l) : { tagName: tagName };
             if(newL && filter(newL, l)) {
-              allLinks[allLinks.length] = newL;
+              allLinks.push(newL);
             }
           }
         }
       } catch(ex) {}
     }
     
-    
-    
+
     filterLinks(doc.links, filter, "A"); 
     
     var uriResolver = null;
     
     if(includeImages) {
-      filterLinks(doc.images, wrapAndFilter,"IMG");
+      filterLinks(doc.images, wrapAndFilter, "IMG");
     }
     
-    filterLinks(doc.embeds,wrapAndFilter,"EMBED");
+    filterLinks(doc.embeds,wrapAndFilter, "EMBED");
 
     return allLinks;
   }
@@ -899,7 +1127,8 @@ var gFlashGot = {
     var isAnchor = (node instanceof HTMLAnchorElement);
     if(isAnchor || node instanceof HTMLAreaElement) {
       var href = node.href;
-      if(href) return { 
+      if(href) {
+        var l = { 
         href: href,
         tagName: "A",
         getElementsByTagName: function(n) { return node.getElementsByTagName(n) },
@@ -907,6 +1136,9 @@ var gFlashGot = {
           ? node.title || node.textContent
           : node.alt || node.title 
         };
+        if ("download" in node) l.fname = node.download;
+        return l;
+      }
     }
     return null;
   }
@@ -1029,9 +1261,13 @@ var gFlashGot = {
   }
 ,
   downloadMedia: function(mediaLinks, dmName) {
+    if (gFlashGotService.getPref("media.autoCloseDocument", false)) {
+      var w = content;
+      w.setTimeout(function() { w.close() }, 500);
+    }
     var links = [];
     return this.addMediaLinks(links, mediaLinks) &&
-      this.download(links, gFlashGotService.OP_SEL, dmName || gFlashGotService.getPref("media.dm", ""));
+      this.download(links, gFlashGotService.OP_SEL, dmName || gFlashGotService.getUPref("media.dm", ""));
   }
 ,
   download: function(links, opType, dmName) {
