@@ -1,7 +1,7 @@
 /***** BEGIN LICENSE BLOCK *****
 
     FlashGot - a Firefox extension for external download managers integration
-    Copyright (C) 2004-2011 Giorgio Maone - g.maone@informaction.com
+    Copyright (C) 2004-2013 Giorgio Maone - g.maone@informaction.com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@ const NS_BINDING_ABORTED = 0x804b0002;
 const EXTENSION_ID = "{19503e42-ca3c-4c27-b1e2-9cdb2170ee34}";
 const EXTENSION_NAME = "FlashGot";
 const CHROME_NAME = "flashgot";
-const VERSION = "1.4.5";
+const VERSION = "1.5.6.7";
 const SERVICE_NAME = EXTENSION_NAME + " Service";
 const SERVICE_CTRID = "@maone.net/flashgot-service;1";
 const SERVICE_ID = "{2a55fc5c-7b31-4ee1-ab15-5ee2eb428cbe}";
@@ -39,7 +39,8 @@ const SERVICE_IIDS =
   CI.nsISupports,
   CI.nsISupportsWeakReference,
   CI.nsIObserver,
-  CI.nsIURIContentListener
+  CI.nsIURIContentListener,
+  CI.nsIContentPolicy,
 ];
 
 // categories which this component is registered in
@@ -47,11 +48,15 @@ const SERVICE_CATS = ["app-startup"];
 
 const IOS = CC["@mozilla.org/network/io-service;1"].getService(CI.nsIIOService);
 
+function B8(n) { // needed because base 8 literals have been deprecated
+  return parseInt(n, 8);
+}
+
 const IO = {
   readFile: function(file) {
     const is = CC["@mozilla.org/network/file-input-stream;1"]
       .createInstance(CI.nsIFileInputStream );
-    is.init(file ,0x01, 0400, null);
+    is.init(file, 0x01, B8('400'), null);
     const sis = CC["@mozilla.org/scriptableinputstream;1"]
       .createInstance(CI.nsIScriptableInputStream );
     sis.init(is);
@@ -71,7 +76,7 @@ const IO = {
     content = unicodeConverter.ConvertFromUnicode(content);
     const os = CC["@mozilla.org/network/file-output-stream;1"]
       .createInstance(CI.nsIFileOutputStream);
-    os.init(file, 0x02 | 0x08 | 0x20, 0700, 0);
+    os.init(file, 0x02 | 0x08 | 0x20, B8('700'), 0);
     os.write(content, content.length);
     os.flush();
     os.close();
@@ -143,7 +148,8 @@ fg = {
     os.addObserver(this, "xpcom-shutdown", false);
     os.addObserver(this, STARTUP, false);
     os.addObserver(this, "private-browsing", false);
-     os.addObserver(this, "browser:purge-session-history", false);
+    os.addObserver(this, "browser:purge-session-history", false);
+    os.addObserver(this, "last-pb-context-exited", false);
   },
   unregister: function() {
     try {
@@ -154,6 +160,7 @@ fg = {
       os.removeObserver(this, STARTUP);
       os.removeObserver(this, "private-browsing");
       os.removeObserver(this, "browser:purge-session-history");
+      os.removeObserver(this, "last-pb-context-exited");
     } catch(ex) {
       this.log("Error unregistering service as observer: " + ex);
     }
@@ -203,10 +210,28 @@ fg = {
           this._shredTempFiles();
           this._erasePrivatePrefs();
         break;
+        case "last-pb-context-exited":
+          this._shredTempFiles();
+        break;
       }
     }
   },
   
+  privatize: function(channel) {
+    if (("nsIPrivateBrowsingChannel" in CI) && channel instanceof CI.nsIPrivateBrowsingChannel) {
+      channel.setPrivate(true);
+    } else {
+      channel.loadFlags |= channel.INHIBIT_PERSISTENT_CACHING;
+    }
+  },
+  
+  isPrivate: function(window) {
+    var privacyContext = window && window.QueryInterface(CI.nsIInterfaceRequestor)
+      .getInterface(CI.nsIWebNavigation);
+    return ((privacyContext instanceof CI.nsILoadContext) && "usePrivateBrowsing" in privacyContext) 
+            ? privacyContext.usePrivateBrowsing
+            : this.inPrivate;
+  },
   
   inPrivate: false,
   uninstalling: false
@@ -235,7 +260,7 @@ fg = {
   }
 ,
   _erasePrivatePrefs: function() {
-    const names = ["redir.rapidshare_com.cookie"];
+    const names = ["redir.rapidshare_com.cookie", "recentDirs"];
     const prefs = this.prefs;
     for each (var n in names) {
       try {
@@ -386,6 +411,9 @@ fg = {
       :this._httpServer;
   }
 ,
+  sanitizeFileName: function(s) {
+    return s.replace(/[\u0000-\u001f\/\\":<>|?*]/g, '_');
+  },
   createDownloadFile: function(parentFile, fileSpec, overwrite) {
     
     if (fileSpec instanceof CI.nsIURL) {
@@ -396,20 +424,26 @@ fg = {
       } catch(e) {
         fileSpec = unescape(fileSpec);
       }
-      fileSpec = fileSpec.replace(/[\u0000-\u001f\/]/g, '_');
     } else if (fileSpec instanceof CI.nsIURI) {
       return null;
     }
     
     if (!fileSpec) return null;
     
+    fileSpec = this.sanitizeFileName(fileSpec);
+    
     var file = parentFile.clone();
     file.append(fileSpec);
             
     if (!overwrite || file.exists() && file.isDirectory()) {
       for (;;) {
         if(!file.exists()) {
-          file.create(0, 0644);
+          try {
+            file.create(0, B8('644'));
+          } catch (e) {
+            this.showFileWriteError(file, e);
+            throw e;
+          }
           break;
         } else { // rename
           var m = file.leafName.match(/(.*?)(?:\((\d+)\))?(\.[^\.]+$|$)/);
@@ -419,6 +453,12 @@ fg = {
     }
     return file;
   }
+,
+  showFileWriteError: function(file, e) {
+    CC["@mozilla.org/embedcomp/prompt-service;1"].getService(CI.nsIPromptService)
+              .alert(null, "FlashGot", this.getString("cannotWriteFile", [file.path]));
+    this.log("ERROR: " + file.path + " cannot be written - " + (e || " unspecificed error"));
+  }
 ,
   download: function(links, opType, dmName) {
     
@@ -432,8 +472,20 @@ fg = {
         if (!opType) opType = this.OP_SEL;
     }
     
+    
+    var win = links.document && links.document.defaultView ||
+      DOM.mostRecentBrowserWindow;
+    if (win && win.content) win = win.content;
+    
+    var privacyContext = win && win.QueryInterface(CI.nsIInterfaceRequestor)
+      .getInterface(CI.nsIWebNavigation);
+    if ((privacyContext instanceof CI.nsILoadContext) && "usePrivateBrowsing" in privacyContext) {
+      this.inPrivate = privacyContext.usePrivateBrowsing;
+    }
+    links.privacyContext = privacyContext || null;
+    
     if (!dmName) dmName = this.defaultDM;
-    const dm = this.DMS[dmName];
+    const dm = this.DMS[dmName] || this.DMS[this.getString("dm.builtIn")];
     if (!dm) {
       this.log("FlashGot error: no download manager selected!");
       return false;
@@ -453,13 +505,6 @@ fg = {
   },
   
   _downloadDelayed: function(links, opType, dm) {
-    
-     if (!links.postData) { 
-      links.postData = null;
-    } else if(!dm.postSupport) {
-      // surrogate POST parameters as query string
-      links[0].href += (links[0].href.indexOf("?") > -1 ?  "&" : "?") + links.postData;
-    }
 
     const encodedURLs = this.getPref(dm.getPref("encode"), this.getPref("encode", true));
 
@@ -507,9 +552,15 @@ fg = {
     var cs = links.document && links.document.characterSet || null;
     
     var j, l, href, ol, pos1, pos2;
+    var postData;
     for (j = 0; j < len; j++) {
       l = links[j];
-      
+      postData =  l.postData || links.postData;
+      if (postData && !dm.postSupport) {
+        // surrogate POST parameters as query string
+        l.href += (l.href.indexOf("?") > -1 ?  "&" : "?") + postData;
+      }
+
       href = l.href;
       
       if (/^javascript:/i.test(href)) {
@@ -524,6 +575,7 @@ fg = {
       if (filters && !filters.doFilter(href)) continue;
       
       if (l.description) l.description = l.description.replace(/\s+/g, ' ');
+      if (l.fname) l.fname = this.sanitizeFileName(l.fname);
       l._pos = j;
       
       ol = map[href];
@@ -756,6 +808,9 @@ fg = {
   }
 ,
   yield: function() {
+    const pref = "dom.max_chrome_script_run_time";
+    var max = this.prefService.getIntPref(pref);
+    if (max < 40) this.prefService.setIntPref(pref, 40);
     try {
       const eqs = CI.nsIEventQueueService;
       if (eqs) {
@@ -766,13 +821,17 @@ fg = {
         const curThread = CC["@mozilla.org/thread-manager;1"].getService().currentThread;
         while (curThread.hasPendingEvents()) curThread.processNextEvent(false);
       }
-    } catch(e) {}
+    } catch(e) {
+    } finally {
+      if (max < 40) this.prefService.setIntPref(pref, max);
+    }
   }
 ,
   
   get prefService() {
     delete this.prefService;
-    return this.prefService = CC["@mozilla.org/preferences-service;1"].getService(CI.nsIPrefService);
+    return this.prefService = CC["@mozilla.org/preferences-service;1"]
+      .getService(CI.nsIPrefService).QueryInterface(CI.nsIPrefBranch);
   }
 ,
   savePrefs: function() {
@@ -797,18 +856,35 @@ fg = {
 ,
   setPref: function(name, value) {
     const prefs = this.prefs;
-    switch (typeof(value)) {
-      case "string":
-          prefs.setCharPref(name,value);
+    try {
+      switch (typeof(value)) {
+        case "string":
+            prefs.setCharPref(name,value);
+            break;
+        case "boolean":
+          prefs.setBoolPref(name,value);
           break;
-      case "boolean":
-        prefs.setBoolPref(name,value);
-        break;
-      case "number":
-        prefs.setIntPref(name,value);
-        break;
-      default:
-        throw new Error("Unsupported type "+typeof(value)+" for preference "+name);
+        case "number":
+          prefs.setIntPref(name,value);
+          break;
+        default:
+          throw new Error("Unsupported type " + typeof(value) + " for preference " + name);
+      }
+    } catch(e) {
+      const IPC = Ci.nsIPrefBranch;
+      try {
+        switch (prefs.getPrefType(name)) {
+          case IPC.PREF_STRING:
+            prefs.setCharPref(name, value);
+            break;
+          case IPC.PREF_INT:
+            prefs.setIntPref(name, parseInt(value));
+            break;
+          case IPC.PREF_BOOL:
+            prefs.setBoolPref(name, !!value && value != "false");
+            break;
+        }
+      } catch(e2) {}
     }
   }
 ,
@@ -839,22 +915,39 @@ fg = {
     var logFile = this.profDir.clone();
     logFile.append("flashgot.log");
     return this.logFile = logFile;
+  },
+  get logURI() {
+    delete this.logURI;
+    return this.logURI = IOS.newFileURI(this.logFile);
+  },
+  logWatch: function() {
+    if (!this.shouldLoad) {
+      this.shouldLoad = function(aContentType, aContentLocation) {
+        if (aContentType === 6 && this.logStream && this.logURI.equals(aContentLocation)) {
+          if (this.logStream) this.logFlush();
+        }
+        return 1;
+      }
+      this.shouldProcess = function() { return 1 };
+      Module.categoryManager.addCategoryEntry("content-policy",
+          SERVICE_CTRID, SERVICE_CTRID, false, true)
+    }
   }
 ,
-  logStream: null,
+
   logEnabled: false,
   logInit: function() {
     const logFile = this.logFile;
     const stream = CC["@mozilla.org/network/file-output-stream;1"]
       .createInstance(CI.nsIFileOutputStream);
-    stream.init(logFile, 0x02 | 0x08 | 0x10, 0600, 0);
+    stream.init(logFile, 0x02 | 0x08 | 0x10, B8('600'), 0);
     const bufferedStream = CC["@mozilla.org/network/buffered-output-stream;1"]
       .createInstance(CI.nsIBufferedOutputStream);
     bufferedStream.init(stream, 32768);
     const charStream = CC["@mozilla.org/intl/converter-output-stream;1"]
       .createInstance(CI.nsIConverterOutputStream);
     charStream.init(bufferedStream, "UTF-8", 0, 0);
-    charStream.writeString("*** FlashGot " + this.VERSION + " started at " + new Date().toGMTString() + " ***\n");
+    charStream.writeString("*** FlashGot " + this.VERSION + " started at " + new Date().toISOString() + " ***\n");
     this.logStream = charStream;
     this.logFlush = function() {
       if (this.logStream) {
@@ -891,9 +984,13 @@ fg = {
   log: function(msg) {
     if (this.logEnabled) {
       try {
-        if (msg !== null && !this.inPrivate) {
-          msg += "\n";
-          this.logStream.writeString(msg);
+        if (msg !== null) {
+          msg = new Date().toISOString() + " " + msg + "\n";
+          if (this.inPrivate) {
+            dump("[FlashGot] " + msg + "\n");
+          } else {
+            this.logStream.writeString(msg);
+          }
         }
 
       } catch(ex) {
@@ -953,9 +1050,9 @@ fg = {
       
       t.append("flashgot." + encodeURI(fg.profDir.leafName).replace(/%/g,"_"));
       if (t.exists()) {
-       if (!(t.isDirectory() && t.isWritable())) t.createUnique(1, 0700);
+       if (!(t.isDirectory() && t.isWritable())) t.createUnique(1, B8('700'));
       } else {
-        t.create(1, 0700);
+        t.create(1, B8('700'));
       } 
       return t;
     }
@@ -997,7 +1094,7 @@ fg = {
         os.addObserver(MediaSniffer, "http-on-examine-response", true);
         os.addObserver(MediaSniffer, "http-on-examine-cached-response", true);
         CC['@mozilla.org/docloaderservice;1'].getService(CI.nsIWebProgress)
-          .addProgressListener(MediaSniffer, CI.nsIWebProgress.NOTIFY_STATE_NETWORK);
+          .addProgressListener(MediaSniffer, CI.nsIWebProgress.NOTIFY_STATE_NETWORK | CI.nsIWebProgress.NOTIFY_LOCATION);
       }
       this.interceptor.setup();
     
@@ -1201,7 +1298,7 @@ fg = {
           logFile.copyTo(logBak.parent, logBak.leafName);
           const is=CC['@mozilla.org/network/file-input-stream;1'].createInstance(
             CI.nsIFileInputStream);
-          is.init(logBak, 0x01, 0400, null);
+          is.init(logBak, 0x01, B8('400'), null);
           is.QueryInterface(CI.nsISeekableStream);
           is.seek(CI.nsISeekableStream.NS_SEEK_END,-maxLogSize);
           const sis=CC['@mozilla.org/scriptableinputstream;1'].createInstance(
@@ -1221,7 +1318,7 @@ fg = {
           if (logStart > -1) {
              const os=CC["@mozilla.org/network/file-output-stream;1"].createInstance(
               CI.nsIFileOutputStream);
-            os.init(logFile,0x02 | 0x08 | 0x20, 0700, 0);
+            os.init(logFile,0x02 | 0x08 | 0x20, B8('700'), 0);
             os.write(content,content.length);
             while ((buffer = sis.read(20000))) {
               os.write(buffer,buffer.length);
@@ -1256,7 +1353,14 @@ fg = {
   getMedia: function(w) {
     return "_flashgotMedia" in w ? w._flashgotMedia : null;
   }
-, 
+,
+  extractFileName: function(channel) {
+    try {
+      return (channel instanceof CI.nsIHttpChannel) &&
+        channel.getResponseHeader("content-disposition").match(/;\s*filename="([^"]+)/i)[1];
+    } catch (e) {}
+    return null;
+  },
 
   firstRun: false,
   versionChecked: false,
@@ -1270,7 +1374,7 @@ fg = {
     if ((this.firstRun = prevVer !== ver)) {
        if (!prevVer) {
         // first run ever
-        this.setPref("media.dm", this.getString("dm.builtIn"));
+        this.setUPref("media.dm", this.getString("dm.builtIn"));
       }
       this.setPref("version", ver);
       this.savePrefs();
@@ -1314,3 +1418,6 @@ fg = {
 }
 
 fg.__global__ = this;
+fg._e = function(f) {
+  return eval("(" + f + ")()"); 
+}
\ No newline at end of file
